import os

from xesam.ontology import *


class StrigiOntoParser (KeyFileParser):
	def __init__ (self):
		KeyFileParser.__init__(self)
		self.fields = {}
		self._current_field = None
		self._current_name = None
		
	def parse (self, dirname, parent_dir=""):
		dirname = os.path.join(parent_dir, dirname)
		if not os.path.exists (dirname):
			raise IOError ("No such directory %s" % dirname)
		if os.path.isdir (dirname):
			for f in os.listdir (dirname):
				self.parse (f, dirname)
		elif dirname.endswith(".fieldproperties"):
			xesam.debug ("Parsing file %s" % dirname)
			KeyFileParser.parse (self, dirname)
		else:
			xesam.debug ("Skipping unknown file %s" % dirname)
		
		if parent_dir == "":
			# This is the root call to the parse method
			self._resolve_relations()
			return self.fields.itervalues()
	
	def got_group (self, group):
		if not group == "Field":
			xesam.error ("Unknown group header: " + group)
			return
		if not self._current_name is None:					
			self._create_field()
			
		self._current_field = {}
		self._current_name = None
	
	def got_key (self, key, val):
		if key == "Uri":
			self._current_name = val
		elif key == "TypeUri":
			self._current_field["Type"] = val
		elif key == "ParentUri":
			self._current_field["Parent"] = val
		elif key == "Description":
			self._current_field["Description"] = val
		elif key == "Name":
			self._current_field["Name"] = val
		else :
			# Just accept the property as is
			self._current_field[key] = val
	
	def _create_field (self):
		item = Item (self._current_name, Item.TYPE_FIELD)
		if not self._current_field.has_key ("Parent"):
			# This must be a root element
			self._current_field["Parent"] = None
		try:		
			self._current_field.pop("Name")
		except KeyError, e:
			xesam.error (str(self._current_field))
			raise OntologyError ("Failed to create field '%s', KeyError: %s" % (self._current_name, e))
			
		item.props = self._current_field
				
		self.fields[self._current_name] = item

	def _resolve_relations (self):
		"""
		Compile and resolve parent-child relationships.
		This will boil the self.fields dict down to a set
		of root Items.
		"""
		to_remove = []
		
		field_count = len(self.fields)
		
		xesam.debug ("Resolving parent-child relations")
		for field in self.fields.itervalues() :
			if not field.props["Parent"] is None:
				if self.fields.has_key(field.props["Parent"]) :
					field.set_parent (self.fields[field.props["Parent"]])
				else:
					xesam.error ("In %s. Cannot resolve parent %s" % (field.get_id(), field.props["Parent"]))
				
			field.props.pop("Parent")
			
			if not isinstance (field.get_parent(), RootItem) :
				to_remove.append(field.get_id())
		
		for non_root_id in to_remove:
			self.fields.pop(non_root_id)
		
		xesam.debug ("Found %s root elements in %s fields" % (len(self.fields), field_count))
